@@ -7,19 +7,35 @@
  *
  * (C) 1999, 2000 Ximian, Inc.
  */
+
+/* FIXME: this needs re-writing to use BonoboObject ! */
+
 #include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <bonobo/Bonobo.h>
+#include <bonobo/bonobo-control.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-canvas-item.h>
 #include <bonobo/bonobo-object.h>
+#include <bonobo/bonobo-types.h>
 #include <bonobo/bonobo-main.h>
+#include <glib/gi18n.h>
+#include <libgnome/gnome-macros.h>
 #include <gdk/gdkprivate.h>
+#if defined (GDK_WINDOWING_X11)
 #include <gdk/gdkx.h>
+#elif defined (GDK_WINDOWING_WIN32)
+#include <gdk/gdkwin32.h>
+#else
+#error Port to this GDK backend
+#endif
 #include <gtk/gtksignal.h>
 
-static GnomeCanvasItemClass *gbi_parent_class;
+GNOME_CLASS_BOILERPLATE (BonoboCanvasItem,
+			 bonobo_canvas_item,
+			 GObject,
+			 GNOME_TYPE_CANVAS_ITEM)
 
 typedef struct {
 	POA_Bonobo_Canvas_ComponentProxy proxy_servant;
@@ -35,9 +51,9 @@ struct _BonoboCanvasItemPrivate {
 };
 
 enum {
-	ARG_0,
-	ARG_CORBA_FACTORY,
-	ARG_CORBA_UI_CONTAINER,
+	PROP_0,
+	PROP_CORBA_FACTORY,
+	PROP_CORBA_UI_CONTAINER
 };
 
 /*
@@ -156,7 +172,8 @@ prepare_state (GnomeCanvasItem *item, Bonobo_Canvas_State *target)
 }
 
 static void
-gbi_update (GnomeCanvasItem *item, double *item_affine, ArtSVP *item_clip_path, int item_flags)
+gbi_update (GnomeCanvasItem *item, double *item_affine,
+	    ArtSVP *item_clip_path, int item_flags)
 {
 	Gbi *gbi = GBI (item);
 	Bonobo_Canvas_affine affine;
@@ -170,8 +187,8 @@ gbi_update (GnomeCanvasItem *item, double *item_affine, ArtSVP *item_clip_path,
 	if (getenv ("DEBUG_BI"))
 		g_message ("gbi_update");
 
-	if (gbi_parent_class)
-		(*gbi_parent_class->update)(item, item_affine, item_clip_path, item_flags);
+	GNOME_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, update,
+			   (item, item_affine, item_clip_path, item_flags));
 	
 	for (i = 0; i < 6; i++)
 		affine [i] = item_affine [i];
@@ -182,6 +199,7 @@ gbi_update (GnomeCanvasItem *item, double *item_affine, ArtSVP *item_clip_path,
 
 	CORBA_exception_init (&ev);
 	prepare_state (item, &state);
+
 	cuta = Bonobo_Canvas_Component_update (
 		gbi->priv->object,
 		&state, affine, clip_path, item_flags,
@@ -190,6 +208,7 @@ gbi_update (GnomeCanvasItem *item, double *item_affine, ArtSVP *item_clip_path,
 
 	if (!BONOBO_EX (&ev)){
 		if (cuta->width > 0 && cuta->height > 0){
+
 			ArtUta *uta;
 
 			uta = uta_from_cuta (cuta);
@@ -229,28 +248,43 @@ static void
 gbi_realize (GnomeCanvasItem *item)
 {
 	Gbi *gbi = GBI (item);
+	Bonobo_Gdk_WindowId id;
+
 	CORBA_Environment ev;
 
 	if (getenv ("DEBUG_BI"))
 		g_message ("gbi_realize");
 	
-	if (gbi_parent_class)
-		(*gbi_parent_class->realize) (item);
+	GNOME_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, realize, (item));
 
-	if (gbi->priv->object == CORBA_OBJECT_NIL){
+	if (gbi->priv->object == CORBA_OBJECT_NIL) {
 		gbi->priv->realize_pending = 1;
 		return;
 	}
+
+        proxy_size_allocate (
+                item->canvas,
+                &(GTK_WIDGET (item->canvas)->allocation),
+                BONOBO_CANVAS_ITEM(item));
 		
 	g_signal_connect (item->canvas, "size_allocate",
 			  G_CALLBACK (proxy_size_allocate), item);
 
 	CORBA_exception_init (&ev);
 	gdk_flush ();
-	Bonobo_Canvas_Component_realize (
-		gbi->priv->object, 
-		GDK_WINDOW_XWINDOW (item->canvas->layout.bin_window),
-		&ev);
+
+#if defined (GDK_WINDOWING_X11)
+	id = bonobo_control_window_id_from_x11
+		(GDK_WINDOW_XWINDOW (item->canvas->layout.bin_window));
+#elif defined (GDK_WINDOWING_WIN32)
+	id = bonobo_control_window_id_from_x11
+		((guint32) GDK_WINDOW_HWND (item->canvas->layout.bin_window));
+#endif
+
+	Bonobo_Canvas_Component_realize (gbi->priv->object, id, &ev);
+
+	CORBA_free (id);
+
 	CORBA_exception_free (&ev);
 }
 
@@ -269,8 +303,7 @@ gbi_unrealize (GnomeCanvasItem *item)
 		CORBA_exception_free (&ev);
 	}
 
-	if (gbi_parent_class)
-		(*gbi_parent_class->unrealize) (item);
+	GNOME_CALL_PARENT (GNOME_CANVAS_ITEM_CLASS, unrealize, (item));
 }
 
 static void
@@ -279,6 +312,7 @@ gbi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	Gbi *gbi = GBI (item);
 	CORBA_Environment ev;
 	Bonobo_Canvas_State state;
+	Bonobo_Gdk_WindowId id;
 	
 	if (getenv ("DEBUG_BI"))
 		g_message ("draw: %d %d %d %d", x, y, width, height);
@@ -290,12 +324,21 @@ gbi_draw (GnomeCanvasItem *item, GdkDrawable *drawable, int x, int y, int width,
 	CORBA_exception_init (&ev);
 
 	prepare_state (item, &state);
+#if defined (GDK_WINDOWING_X11)
+	id = bonobo_control_window_id_from_x11
+		(GDK_WINDOW_XWINDOW (drawable));
+#elif defined (GDK_WINDOWING_WIN32)
+	id = bonobo_control_window_id_from_x11
+		((guint32) GDK_WINDOW_HWND (drawable));
+#endif
+	
 	Bonobo_Canvas_Component_draw (
 		gbi->priv->object,
 		&state,
-		GDK_WINDOW_XWINDOW (drawable),
-		x, y, width, height,
+		id, x, y, width, height,
 		&ev);
+
+	CORBA_free (id);
 	CORBA_exception_free (&ev);
 }
 
@@ -439,7 +482,7 @@ gdk_event_to_bonobo_event (GdkEvent *event)
 		e->_d = Bonobo_Gdk_MOTION;
 		e->_u.motion.time = event->motion.time;
 		e->_u.motion.x = event->motion.x;
-		e->_u.motion.y = event->motion.x;
+		e->_u.motion.y = event->motion.y;
 		e->_u.motion.x_root = event->motion.x_root;
 		e->_u.motion.y_root = event->motion.y_root;
 #ifdef FIXME
@@ -483,6 +526,7 @@ gdk_event_to_bonobo_event (GdkEvent *event)
 		e->_u.crossing.y = event->crossing.y;
 		e->_u.crossing.x_root = event->crossing.x_root;
 		e->_u.crossing.y_root = event->crossing.y_root;
+		e->_u.crossing.state = event->crossing.state;
 
 		switch (event->crossing.mode){
 		case GDK_CROSSING_NORMAL:
@@ -531,23 +575,28 @@ gbi_event (GnomeCanvasItem *item, GdkEvent *event)
 }
 
 static void
-gbi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
+gbi_set_property (GObject      *object,
+		  guint         property_id,
+		  const GValue *value,
+		  GParamSpec   *pspec)
 {
-	Gbi *gbi = GBI (o);
+	Gbi *gbi = GBI (object);
 	Bonobo_Canvas_ComponentProxy proxy_ref;
 	Bonobo_CanvasComponentFactory factory;
 	CORBA_Environment ev;
 
-	switch (arg_id){
-	case ARG_CORBA_FACTORY:
+	switch (property_id) {
+	case PROP_CORBA_FACTORY:
 
 		CORBA_exception_init (&ev);
 
-		if (gbi->priv->object)
-			Bonobo_Canvas_Component_unref (gbi->priv->object, &ev);
-		
-		gbi->priv->object = CORBA_OBJECT_NIL;
-		factory = GTK_VALUE_POINTER (*arg);
+		gbi->priv->object = bonobo_object_release_unref (gbi->priv->object, &ev);
+
+                /* FIXME: I can't make it work as corba_object. 
+                   Using bonobo_unknown for now. */
+		/* factory = bonobo_value_get_corba_object (value); */
+               
+		factory = bonobo_value_get_unknown (value);
 
 		g_return_if_fail (factory != CORBA_OBJECT_NIL);
 
@@ -559,35 +608,32 @@ gbi_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 				factory, GNOME_CANVAS_ITEM (gbi)->canvas->aa, 
 				proxy_ref, &ev);
 
+		if (ev._major != CORBA_NO_EXCEPTION)
+			gbi->priv->object = CORBA_OBJECT_NIL;
+
+		CORBA_Object_release (factory, &ev);
+
 		CORBA_exception_free (&ev);
 
 		if (gbi->priv->object == CORBA_OBJECT_NIL) {
-			gtk_object_unref (GTK_OBJECT (gbi));
+			g_object_unref (gbi);
 			return;
 		}
 
-		/* Initial size notification */
-		proxy_size_allocate (GNOME_CANVAS_ITEM (gbi)->canvas,
-			     &(GTK_WIDGET (GNOME_CANVAS_ITEM (gbi)->canvas)->allocation), gbi);
-	
 		if (gbi->priv->realize_pending){
 			gbi->priv->realize_pending = 0;
 			gbi_realize (GNOME_CANVAS_ITEM (gbi));
 		}
 		break;
 
-	case ARG_CORBA_UI_CONTAINER:
-
-		gbi->priv->proxy->ui_container = GTK_VALUE_POINTER (*arg);
+	case PROP_CORBA_UI_CONTAINER:
+		gbi->priv->proxy->ui_container = bonobo_value_get_unknown (value);
 
 		g_return_if_fail (gbi->priv->proxy->ui_container != CORBA_OBJECT_NIL);
-
-		bonobo_object_dup_ref (gbi->priv->proxy->ui_container, NULL);
-
 		break;
 
 	default:
-		g_warning ("Unexpected arg_id %u", arg_id);
+		g_warning ("Unexpected arg_id %u", property_id);
 		break;
 	}
 }
@@ -603,8 +649,7 @@ gbi_finalize (GObject *object)
 
 	CORBA_exception_init (&ev);
 
-	if (gbi->priv->object != CORBA_OBJECT_NIL)
-		Bonobo_Canvas_Component_unref (gbi->priv->object, &ev);
+	bonobo_object_release_unref (gbi->priv->object, &ev);
 
 	if (gbi->priv->proxy){
 		ComponentProxyServant *proxy = gbi->priv->proxy;
@@ -617,30 +662,41 @@ gbi_finalize (GObject *object)
 	
 	g_free (gbi->priv);
 	CORBA_exception_free (&ev);
-	
-	(*G_OBJECT_CLASS (gbi_parent_class)->finalize)(object);
+
+	GNOME_CALL_PARENT (G_OBJECT_CLASS, finalize, (object));
 }
 
 static void
-gbi_class_init (GtkObjectClass *object_class)
+bonobo_canvas_item_class_init (BonoboCanvasItemClass *object_class)
 {
-	GnomeCanvasItemClass *item_class = (GnomeCanvasItemClass *) object_class;
 	GObjectClass *gobject_class = (GObjectClass *) object_class;
+	GnomeCanvasItemClass *item_class =
+		(GnomeCanvasItemClass *) object_class;
 
-	gbi_parent_class = gtk_type_class (gnome_canvas_item_get_type ());
+	gobject_class->set_property = gbi_set_property;
 
-	gtk_object_add_arg_type (
-		"BonoboCanvasItem::corba_factory",
-		GTK_TYPE_POINTER,
-		GTK_ARG_WRITABLE, ARG_CORBA_FACTORY);
+	g_object_class_install_property (
+		gobject_class,
+		PROP_CORBA_FACTORY,
+		g_param_spec_boxed (
+			"corba_factory",
+			_("corba factory"),
+			_("The factory pointer"),
+			BONOBO_TYPE_STATIC_UNKNOWN,
+			G_PARAM_WRITABLE));
+
+	g_object_class_install_property (
+		gobject_class,
+		PROP_CORBA_UI_CONTAINER,
+		g_param_spec_boxed (
+			"corba_ui_factory",
+			_("corba UI container"),
+			_("The User interface container"),
+			BONOBO_TYPE_STATIC_UNKNOWN,
+			G_PARAM_WRITABLE));
 	
-	gtk_object_add_arg_type (
-		"BonoboCanvasItem::corba_ui_container",
-		GTK_TYPE_POINTER,
-		GTK_ARG_WRITABLE, ARG_CORBA_UI_CONTAINER);
-	
-	object_class->set_arg  = gbi_set_arg;
-	gobject_class->finalize = gbi_finalize;
+	gobject_class->finalize     = gbi_finalize;
+
 	item_class->update     = gbi_update;
 	item_class->realize    = gbi_realize;
 	item_class->unrealize  = gbi_unrealize;
@@ -651,27 +707,58 @@ gbi_class_init (GtkObjectClass *object_class)
 	item_class->event      = gbi_event;
 }
 
+static gboolean
+gbi_idle_handler (GnomeCanvasItem *item)
+{
+	gnome_canvas_item_request_update (item);
+        return FALSE;
+}
+
 static void
 impl_Bonobo_Canvas_ComponentProxy_requestUpdate (PortableServer_Servant servant,
-					         CORBA_Environment *ev)
+					         CORBA_Environment     *ev)
 {
 	ComponentProxyServant *item_proxy = (ComponentProxyServant *) servant;
 
-	gnome_canvas_item_request_update (item_proxy->item_bound);
+        if (item_proxy->item_bound->canvas->idle_id == 0)
+        {
+        	gnome_canvas_item_request_update (item_proxy->item_bound);
+        }
+        else
+        {
+                /* Problem: It is possible to get here at a time when the canvas
+                 * is in the middle of a do_update.  This happens because
+                 * this proxy call might be waiting in the queue when
+                 * the this object calls Bonobo_Canvas_Component_update.
+                 *
+                 * Solution:
+                 * The canvas is either in its do_update routine or it is
+                 * waiting to do an update.  If it is waiting this idle_handler
+                 * will get called just before the canvas's handler.  If it is
+                 * in the middle of a do_update, then this will queue for the
+                 * next one.
+                 */
+
+                 g_idle_add_full(GDK_PRIORITY_REDRAW-6,
+                                 (GSourceFunc)gbi_idle_handler, 
+                                 item_proxy->item_bound, NULL);
+        }
 
 }
 					    
 static void
 impl_Bonobo_Canvas_ComponentProxy_grabFocus (PortableServer_Servant servant,
-					     guint32 mask, 
-					     gint32 cursor_type,
-					     guint32 time,
-					     CORBA_Environment *ev)
+					     guint32                mask, 
+					     gint32                 cursor_type,
+					     guint32                time,
+					     CORBA_Environment     *ev)
 {
 	ComponentProxyServant *item_proxy = (ComponentProxyServant *) servant;
 	GdkCursor *cursor;
 
-	cursor = gdk_cursor_new ((GdkCursorType) cursor_type);
+	cursor = gdk_cursor_new_for_display
+		(gtk_widget_get_display (GTK_WIDGET (item_proxy->item_bound->canvas)),
+		 (GdkCursorType) cursor_type);
 
 	gnome_canvas_item_grab (item_proxy->item_bound, mask, cursor, time);
 }
@@ -688,7 +775,7 @@ impl_Bonobo_Canvas_ComponentProxy_ungrabFocus (PortableServer_Servant servant,
 
 static Bonobo_UIContainer
 impl_Bonobo_Canvas_ComponentProxy_getUIContainer (PortableServer_Servant servant,
-						  CORBA_Environment *ev)
+						  CORBA_Environment     *ev)
 {
 	ComponentProxyServant *item_proxy = (ComponentProxyServant *) servant;
 
@@ -698,10 +785,12 @@ impl_Bonobo_Canvas_ComponentProxy_getUIContainer (PortableServer_Servant servant
 	return bonobo_object_dup_ref (item_proxy->ui_container, NULL);
 }
 
+static PortableServer_ServantBase__epv item_proxy_base_epv;
+
 static POA_Bonobo_Canvas_ComponentProxy__epv item_proxy_epv;
 
 static POA_Bonobo_Canvas_ComponentProxy__vepv item_proxy_vepv = {
-	NULL,
+	&item_proxy_base_epv,
 	&item_proxy_epv
 };
 
@@ -715,6 +804,8 @@ create_proxy (GnomeCanvasItem *item)
 	ComponentProxyServant *item_proxy = g_new0 (ComponentProxyServant, 1);
 	CORBA_Environment ev;
 	
+        item_proxy->proxy_servant.vepv = &item_proxy_vepv;
+
 	CORBA_exception_init (&ev);
 	POA_Bonobo_Canvas_ComponentProxy__init ((PortableServer_Servant) item_proxy, &ev);
 
@@ -723,7 +814,6 @@ create_proxy (GnomeCanvasItem *item)
 	item_proxy_epv.ungrabFocus    = impl_Bonobo_Canvas_ComponentProxy_ungrabFocus;
 	item_proxy_epv.getUIContainer = impl_Bonobo_Canvas_ComponentProxy_getUIContainer;
 
-	item_proxy->proxy_servant.vepv = &item_proxy_vepv;
 	item_proxy->item_bound = item;
 
 	item_proxy->oid = PortableServer_POA_activate_object (
@@ -735,42 +825,15 @@ create_proxy (GnomeCanvasItem *item)
 }
 
 static void
-gbi_init (BonoboCanvasItem *gbi)
+bonobo_canvas_item_instance_init (BonoboCanvasItem *gbi)
 {
 	gbi->priv = g_new0 (BonoboCanvasItemPrivate, 1);
 	gbi->priv->proxy = create_proxy (GNOME_CANVAS_ITEM (gbi));
 }
 
-/**
- * bonobo_canvas_item_get_type:
- *
- * Returns the GtkType associated with a #BonoboCanvasItem canvas item
- */
-GtkType
-bonobo_canvas_item_get_type (void)
-{
-	static GtkType type = 0;
-
-	if (!type) {
-		GtkTypeInfo info = {
-			"BonoboCanvasItem",
-			sizeof (BonoboCanvasItem),
-			sizeof (BonoboCanvasItemClass),
-			(GtkClassInitFunc) gbi_class_init,
-			(GtkObjectInitFunc) gbi_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
-		};
-
-		type = gtk_type_unique (gnome_canvas_item_get_type (), &info);
-	}
-
-	return type;
-}
-
 void
-bonobo_canvas_item_set_bounds (BonoboCanvasItem *item, double x1, double y1, double x2, double y2)
+bonobo_canvas_item_set_bounds (BonoboCanvasItem *item, double x1, double y1,
+			       double x2, double y2)
 {
-	
+	g_warning ("Unimplemented");
 }
