@@ -49,7 +49,8 @@ bonobo_dock_item_grip_expose (GtkWidget      *widget,
 			  shadow,
 			  clip, widget, "dockitem",
 			  rect->x, rect->y, rect->width, rect->height, 
-			  grip->item->orientation);
+			  (grip->item->orientation == GTK_ORIENTATION_HORIZONTAL)
+			    ? GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL);
 
 	if (GTK_WIDGET_HAS_FOCUS (widget)) {
 		gint focus_width;
@@ -127,6 +128,7 @@ static void
 bonobo_dock_item_grip_dock (BonoboDockItemGrip *grip)
 {
 	BonoboDock *dock;
+	int placement;
 
 	g_return_if_fail (BONOBO_IS_DOCK_ITEM_GRIP (grip));
 
@@ -143,16 +145,22 @@ bonobo_dock_item_grip_dock (BonoboDockItemGrip *grip)
 		GTK_CONTAINER (
 			GTK_WIDGET (grip->item)->parent),
 		GTK_WIDGET (grip->item));
+
+	if (grip->item->orientation == GTK_ORIENTATION_HORIZONTAL)
+		placement = BONOBO_DOCK_TOP;
+	else
+		placement = BONOBO_DOCK_LEFT;
+
 	bonobo_dock_add_item (
 		dock, grip->item,
-		BONOBO_DOCK_TOP, 2, 0, 0, TRUE);
-	g_object_unref (G_OBJECT (grip->item));
+		placement, 2, 0, 0, TRUE);
+	g_object_unref (grip->item);
 }
 
 static void
 bonobo_dock_item_grip_undock (BonoboDockItemGrip *grip)
 {
-	guint x, y;
+	int x, y;
 
 	g_return_if_fail (BONOBO_IS_DOCK_ITEM_GRIP (grip));
 
@@ -247,7 +255,7 @@ bonobo_dock_item_grip_get_accessible (GtkWidget *widget)
 		bonobo_a11y_add_actions_interface (
 			a11y_type, &action_if,
 			ACTION_DOCK,   "dock",   _("Dock the toolbar"),    "<Enter>",
-			ACTION_UNDOCK, "undock", _("Un dock the toolbar"), "<Enter>",
+			ACTION_UNDOCK, "undock", _("Undock the toolbar"), "<Enter>",
 			-1);
 	}
 
@@ -270,7 +278,7 @@ bonobo_dock_item_grip_activate (BonoboDockItemGrip *grip)
 static void
 bonobo_dock_item_grip_instance_init (BonoboDockItemGrip *grip)
 {
-/*	GTK_WIDGET_SET_FLAGS (grip, GTK_CAN_FOCUS); */
+	GTK_WIDGET_SET_FLAGS (grip, GTK_CAN_FOCUS); 
 	GTK_WIDGET_SET_FLAGS (grip, GTK_NO_WINDOW);
 }
 
